<?php
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Author: Laurent Laville <laurent.laville@worldonline.fr>             |
// +----------------------------------------------------------------------+
//
// $Id: Progress.php,v 1.2 2003/07/27 13:25:39 Farell Exp $

require_once ('PEAR.php');
require_once ('HTML/CSS.php');
require_once ('HTML/Page.php');

/**
 * The HTML_Progress class allow you to add a loading bar
 * to any of your html document.
 * You should have a browser that accept DHTML feature.
 *
 * @version    0.4.2
 * @author     Laurent Laville <laurent.laville@worldonline.fr>
 * @access     public
 * @package    HTML_Progress
 */

class HTML_Progress {

    /**
     * Current progress percent 
     *
     * @var        integer
     * @since      0.2
     * @access     public
     */
    var $progress;

    /**
     * Progress_Bar size (width)
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_width;

    /**
     * Progress_Bar size (height)
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_height;

    /**
     * Progress_Bar background color
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_bgcolor;

    /**
     * Progress_Bar padding to border
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_padding;

    /**
     * Progress_Bar border (size, style, color)
     *
     * @var        array
     * @since      0.1
     * @access     public
     */
    var $bar_border = array();

    /**
     * Progress_Bar cell size (width)
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_cell_width;

    /**
     * Progress_Bar cell size (height)
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_cell_height;

    /**
     * Progress_Bar active cell color
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_cell_active;

    /**
     * Progress_Bar inactive cell color
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_cell_inactive;

    /**
     * Progress_Bar space between each cell
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_cell_space;

    /**
     * Progress_Bar percent text info SHOW or HIDE
     *
     * @var        boolean
     * @since      0.2
     * @access     public
     */
    var $bar_text_visible;

    /**
     * Progress_Bar percent text info (font family)
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_text_font;
    
    /**
     * Progress_Bar percent text info (font size)
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_text_size;

    /**
     * Progress_Bar percent text info (font color)
     *
     * @var        string
     * @since      0.1
     * @access     public
     */
    var $bar_text_color;
  
    /**
     * Parent Page of progress bar
     *
     * @var        object
     * @since      0.1
     * @access     public
     */
    var $_page;
  
    /**
     * Contains the line end string
     *
     * @var        string
     * @since      0.2
     * @access     private
     */
    var $_lineEnd;


    /**
     * Class constructor
     *
     * @param      mixed     $self          (optional) Progress Bar is integrated on HTML Page (current)
     * @param      array     $attributes    (optional) Associative array of HTML tag attributes
     * @param      string    $script        (optional) URL to the linked Progress JavaScript            
     *
     * @since      0.1
     * @access     public
     */
    function HTML_Progress($self=false, $attributes=array(), $script='progress.js')
    {
        // Define Line End string
        $style = (OS_WINDOWS) ? 'win' : 'unix';
        HTML_Common::setLineEnd($style);
        $this->_lineEnd = HTML_Common::_getLineEnd();
        
        if ( is_object($self) &&
            (get_class($self) == 'html_page' || is_subclass_of($self, 'html_page'))) {
            $this->_page =& $self; 
        } else {
            $this->_page = new HTML_Page(array ( 
                           'lineend'   => $style,
                           'doctype'   => 'XHTML 1.0 Strict',
                           'language'  => 'en',
                           'cache'     => 'false'
            ));
            $this->_page->setTitle("PEAR::HTML_Progress");
            $this->_page->setMetaData("author", "Laurent Laville");
        }
        $this->_page->addScript($script);

        $this->bar_width   = isset($attributes['width']) ? $attributes['width'] : "180px";
        $this->bar_height  = isset($attributes['height']) ? $attributes['height'] : "22px";
        $this->bar_bgcolor = isset($attributes['background-color']) ? $attributes['background-color'] : "white";
        $this->bar_padding = isset($attributes['padding']) ? $attributes['padding'] : "2px";
        
        $this->bar_border['width'] = isset($attributes['border-width']) ? $attributes['border-width'] : "0px";
        $this->bar_border['style'] = isset($attributes['border-style']) ? $attributes['border-style'] : "solid";
        $this->bar_border['color'] = isset($attributes['border-color']) ? $attributes['border-color'] : "black";
        
        // Apply default cell styles
        $this->bar_cell_width    = isset($attributes['cell-width']) ? $attributes['cell-width'] : "15px";
        $this->bar_cell_height   = isset($attributes['cell-height']) ? $attributes['cell-height'] : "20px";
        $this->bar_cell_spacing  = isset($attributes['cell-spacing']) ? $attributes['cell-spacing'] : "2px";
        $this->bar_cell_active   = isset($attributes['active-color']) ? $attributes['active-color'] : "#006600";
        $this->bar_cell_inactive = isset($attributes['inactive-color']) ? $attributes['inactive-color'] : "#CCCCCC";

        // Apply default percent text styles
        $this->setText();

        // to fix a potential php config problem with PHP 4.2.0 : turn 'implicit_flush' ON
        ob_implicit_flush(1);
    }

    /**
     * Returns the current API version
     *
     * @return     float
     * @since      0.1
     * @access     public
     */
    function apiVersion()
    {
        return 0.4;
    }

    /**
     * Set percent number of progress bar
     *
     * @param      integer   $progress      percent number to set or update
     *
     * @return     void
     * @since      0.2
     * @access     public
     */
    function setProgress($progress)
    {
        $this->progress = $progress;
    }

    /**
     * Returns current percent number of progress bar
     *
     * @return     integer                  percent number
     * @since      0.2
     * @access     public
     */
    function getProgress()
    {
        return $this->progress;
    }

    /**
     * Define look and feel of percent text info in progress bar
     *
     * @param      boolean   $visible       (optional) show or hide the percent text info tag
     * @param      array     $attributes    (optional) Associative array of HTML tag attributes
     *
     * @return     void
     * @since      0.1
     * @access     public
     */
    function setText($visible=false, $attributes=array())
    {
        $this->bar_text_visible = $visible;
        
        if ($visible) {
            $this->bar_text_font  = isset($attributes['font']) ? $attributes['font'] : "Verdana, Arial, Helvetica, sans-serif";
            $this->bar_text_size  = isset($attributes['size']) ? $attributes['size'] : "12px";
            $this->bar_text_color = isset($attributes['color']) ? $attributes['color'] : "black";
        }
    }

    /**
     * Generates and returns the complete Progress Bar as a string
     *
     * @return     string                   HTML Progress bar
     * @since      0.2
     * @access     public
     */
    function toHTML()
    {
        $bar = <<< HTML
<div id="progressCell0" class="cell0">&nbsp;</div>
<div id="progressCell1" class="cell0">&nbsp;</div>
<div id="progressCell2" class="cell0">&nbsp;</div>
<div id="progressCell3" class="cell0">&nbsp;</div>
<div id="progressCell4" class="cell0">&nbsp;</div>
<div id="progressCell5" class="cell0">&nbsp;</div>
<div id="progressCell6" class="cell0">&nbsp;</div>
<div id="progressCell7" class="cell0">&nbsp;</div>
<div id="progressCell8" class="cell0">&nbsp;</div>
<div id="progressCell9" class="cell0">&nbsp;</div>
HTML;

        $info = <<< TEXT
<div id="installationProgress">0 %</div>
TEXT;

        $str  = $this->_lineEnd ."<div id=\"progressBar\">". $this->_lineEnd;
        $str .= ($this->bar_text_visible ? $bar.$info : $bar);
        $str .= $this->_lineEnd ."</div>". $this->_lineEnd;

        return $str;
    }

    /**
     * Display progress_bar in current html page
     *
     * @param      integer   $progress      percent step to add, or percent info to set to progress bar
     * @param      string    $method        (optional) method to increase progress bar ('add' or 'set')
     *
     * @return     void
     * @since      0.2
     * @access     public
     */
    function display($progress, $method="add")
    {
        if (!isset($this->progress) && !headers_sent()) {
            $css =& $this->getStyle();
            $this->_page->addStyleDeclaration(&$css);
            $this->_page->addBodyContent($this->toHTML());
            $this->_page->display();
            $this->progress = 0;
        }

        $method = strtolower($method);


        switch ($method) {
         case 'add':
         case 'set':
             $bar  = ob_get_contents();
             $bar .= $this->_lineEnd;
             @ob_end_clean();
             if ($method == 'add') {
                 $this->progress += $progress;
                 $bar .= '<script type="text/javascript">parent.addprogress('.((int) $progress)."); </script>";
             } else {
                 $this->progress  = $progress;
                 $bar .= '<script type="text/javascript">parent.setprogress('.((int) $progress)."); </script>";
             }
             echo $bar;
             ob_start();
             sleep(1);
             break;
         default:
        }
    }

    /**
     * Get the custom styles sheet to put inline on HTML document
     *
     * @return     object                   HTML_CSS instance
     * @since      0.2
     * @access     public
     */
    function &getStyle()
    {
        $css = new HTML_CSS();

        $css->setStyle('#progressBar', 'background-color', $this->bar_bgcolor);
        $css->setStyle('#progressBar', 'border-width', $this->bar_border['width']);
        $css->setStyle('#progressBar', 'border-style', $this->bar_border['style']);
        $css->setStyle('#progressBar', 'border-color', $this->bar_border['color']);
        $css->setStyle('#progressBar', 'width', $this->bar_width);
        $css->setStyle('#progressBar', 'height', $this->bar_height);
        $css->setStyle('#progressBar', 'padding', $this->bar_padding);
        if ($this->bar_text_visible) {
            $css->setStyle('#progressBar', 'font-family', $this->bar_text_font);
            $css->setStyle('#progressBar', 'font-size', $this->bar_text_size);
            $css->setStyle('#progressBar', 'color', $this->bar_text_color);
        }

        $css->setStyle('#installationProgress', 'text-align', 'right');

        $progressCells = "#progressCell0";
        for ($i=1; $i<10; $i++) {
            $progressCells .= ",#progressCell".$i;
        }
        $css->setStyle($progressCells, 'float', 'left');
        $css->setStyle($progressCells, 'width', $this->bar_cell_width);
        $css->setStyle($progressCells, 'height', $this->bar_cell_height);
        $css->setStyle($progressCells, 'margin-right', $this->bar_cell_spacing);

        $css->setStyle('.cell0', 'background-color', $this->bar_cell_inactive);

        $css->setStyle('.cell1', 'background-color', $this->bar_cell_active);

        return $css;
    }
}

?>